/* manpath program: (C) TOYODA Eizi, 1995
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "config.h"
#ifdef NEED_MALLOC_H
# include <malloc.h>
#endif
#include DIRHDR

#define	eputs(a)	(void)fputs((a), stderr)
#define streq(x, y) (strcmp((x), (y)) == 0)
#define strhead(x, y) (strncmp((x), (y), strlen(y)) == 0)
#define MALLOC(siz) mymalloc((int)(siz))

#define CHAPTERCHAR			"183lnop24567"
#define SUBDIR_APPEND_LEN	10

char	*AllChapters[] = {
	"1",
#ifdef CHAPTER_ONE_M
	"1m",
#endif
	"8", "6", "l", "n", "o", "p", "3", "2", "4", "5", "7",
	NULL
};

char	*ProgName;
char	*ManPath;
char	*Lang;
int		optT = 0;
int		optQ = 0;

	char *
mymalloc(len)
	int		len;
{
	char	*ans;
	if ((ans = malloc((size_t)len)) == NULL) {
		(void)fputs("malloc() failed!\n", stderr);
		exit(1);
	}
	return ans;
}

	char *
shortchapter(chap)
	char	*chap;
{
	static char		ans[2];
#ifdef CHAPTER_ONE_M
	if (streq(chap, "1m"))
		return chap;
#endif
	ans[1] = '\0';
	ans[0] = *chap;
	return ans;
}

#ifdef NOCHECKEXT
#define testfile(chap, nam, fnam) 1
#else
	int
testfile(chap, nam, fnam)
	char	*chap, *nam, *fnam;
{
	if (strlen(chap) == 1)
		return fnam[strlen(nam)] == *chap;
	return streq(chap, fnam+strlen(nam));
}
#endif

	int
testdir(path, chap, nam, subdirfmt)
	char	*path, *chap, *nam, *subdirfmt;
{
	char			*dirname;
	DIR				*dirp;
	struct dirent	*dinfo;
	int				namlen;
	namlen = strlen(nam);
	dirname = MALLOC(strlen(path)+SUBDIR_APPEND_LEN);
	(void)sprintf(dirname, subdirfmt, path, shortchapter(chap));
#ifdef DEBUG
	printf("@ %s %s %s\n", dirname, chap, nam);
#endif
	if ((dirp = opendir(dirname)) != NULL) {
		for (dinfo=readdir(dirp); dinfo; dinfo=readdir(dirp)) {
			if (strncmp(dinfo->d_name, nam, namlen))
				continue;
			if (testfile(chap, nam, dinfo->d_name)) {
				if (!optQ)
					(void)printf("%s/%s\n", dirname, dinfo->d_name);
				(void)closedir(dirp);
				return 1;
			}
		}
		(void)closedir(dirp);
	}
	return 0;
}

	int
testchapter(path, chap, nam)
	char	*path, *chap, *nam;
{
#ifdef COMPRESSED_MAN
	if (!optT && testdir(path, chap, nam, "%s/cat%s.Z"))
		return 1;
#endif
	if (!optT && testdir(path, chap, nam, "%s/cat%s"))
		return 1;
#ifdef COMPRESSED_MAN
	if (testdir(path, chap, nam, "%s/man%s.Z"))
		return 1;
#endif
	return testdir(path, chap, nam, "%s/man%s");
}

	int
testpath(path, chap, nam)
	char	*path, *chap, *nam;
{
	char	**p;
	if (*chap != '?')
		return testchapter(path, chap, nam);
	for (p=AllChapters; *p; p++) {
		if (testchapter(path, *p, nam))
			return 1;
	}
	return 0;
}

#ifdef NOLANG
# define testLpath(path, chap, nam) testpath(path,chap,nam)
#else
	int
testLpath(path, chap, nam)
	char	*path, *chap, *nam;
{
	char	*pa;
	if (Lang) {
		pa = MALLOC(strlen(path) + strlen(Lang) + 2);
		(void)sprintf(pa, "%s/%s", path, Lang);
		if (testpath(pa, chap, nam)) {
			(void)free(pa);
			return 1;
		}
		(void)free(pa);
	}
	return testpath(path, chap, nam);
}
#endif

	int
page(chap, ptitle)
	char *chap, *ptitle;
{
	char *pa, *nam;
	pa = ManPath;
	nam = MALLOC(strlen(ptitle)+2);
#ifdef NOCHECKEXT
	(void)sprintf(nam, "%s", ptitle);
#else
	(void)sprintf(nam, "%s.", ptitle);
#endif
	do {
		char	*pb, *px;
		int	i;
		pb = strchr(pa, ':');
		i = (pb == NULL) ? (strlen(pa)) : (pb-pa);
		px = strncpy(MALLOC(i+1), pa, i);
		px[i] = '\0';
		if (testLpath(px, chap, nam))
			return 0;
		pa = (pb == NULL) ? NULL : (pb+1);
		(void)free(px);
	} while (pa);
	if (!optQ)
		(void)fprintf(stderr, "%s%s not found\n", nam, chap);
	(void)free(nam);
	return 1;
}

	char *
chapter(opt)
	char *opt;
{
	static char	ans[3];
	char		*p;
	p = ans;
	if (streq(opt, "local")) return "l";
	if (streq(opt, "new")) return "n";
	if (streq(opt, "old")) return "o";
	if (streq(opt, "public")) return "p";
	if (*opt=='\0' || strchr(CHAPTERCHAR, *opt) == NULL)
		return NULL;
	*p++ = *opt++;
	if (isdigit(*ans) && isalpha(*opt))
		*p++ = tolower(*opt++);
	if (*opt)
		return NULL;
	*p = '\0';
	return ans;
}

	void
InitEnv()
{
	ManPath = getenv("MANPATH");
	Lang = getenv("LC_MESSAGES");
	if (ManPath == NULL || streq(ManPath, ""))
		ManPath = DEFAULT_MANPATH;
	if (Lang == NULL || streq(Lang, ""))
		Lang = getenv("LANG");
	if (Lang == NULL || streq(Lang, "") || streq(Lang, "C"))
		Lang = NULL;
}

	void
disphelp()
{
	eputs("usage: ");
	eputs(ProgName);
	eputs(" [-htq] [-M path] [-L lang] [section] name ...\n");
	eputs("section: [0-8lnop][a-z] | local | new | old | public\n");
	exit(1);
}

	void
option(opt)
	char	*opt;
{
	if (strhead(opt, "-M")) {
		ManPath = opt+2;
		return;
	} else if (strhead(opt, "-L")) {
		Lang = opt+2;
		return;
	}
	if (strchr(opt, 'h'))
		disphelp();
	if (strchr(opt, 't'))
		optT = 1;
	if (strchr(opt, 'q'))
		optQ = 1;
}

	int
main(argc, argv)
	int	argc;
	char	*argv[];
{
	int		i, r=0, pg=0;
	char	*chap;
	ProgName = argv[0];
	InitEnv();
	for (i=1; i < argc; i++) {
		if (streq(argv[i], "-M")) {
			if (++i >= argc)
				disphelp();
			ManPath = argv[i];
		} else if (streq(argv[i], "-L")) {
			if (++i >= argc)
				disphelp();
			Lang = argv[i];
		} else if (*argv[i] == '-') {
			option(argv[i]);
		} else if ((chap = chapter(argv[i])) == NULL) {
			r = page("?", argv[i]);
			pg++;
		} else {
			if (++i >= argc)
				disphelp();
			r = page(chap, argv[i]);
			pg++;
		}
		if (r)
			break;
	}
	if (pg == 0)
		disphelp();
	return r;
}

